# Copyright (C) 2019 Majormode.  All rights reserved.
#
# This software is the confidential and proprietary information of
# Majormode or one of its subsidiaries.  You shall not disclose this
# confidential information and shall use it only in accordance with the
# terms of the license agreement or other applicable agreement you
# entered into with Majormode.
#
# MAJORMODE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
# OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
# TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE, OR NON-INFRINGEMENT.  MAJORMODE SHALL NOT BE LIABLE FOR ANY
# LOSSES OR DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
# OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.

from majormode.perseus.client.service.base_service import BaseService
from majormode.perseus.model.locale import DEFAULT_LOCALE
from majormode.perseus.model.obj import Object


class ChildService(BaseService):
    BaseService._declare_custom_exceptions({
    })

    def search_children(
            self,
            keywords=None,
            limit=None,
            locale=DEFAULT_LOCALE,
            offset=0):
        """
        Return a list of children that have joined the School Bus Program and
        that correspond to the specified criteria.


        :param keywords: A list of keywords to search children for.

        :param limit: Constrain the number of places to return to the
            specified number.  If not specified, the default value is
            ``BaseService.DEFAULT_RESULT_SET_SIZE``.  The maximum value is
            ``BaseService.MAXIMUM_RESULT_SET_LIMIT``.

        :param locale: An instance ``Locale`` to return textual information,
            such as the names of countries and the names of children.

        :param offset: Require to skip that many records before beginning to
            return records to the client.  Default value is ``0``.  If both
            ``offset`` and ``limit`` are specified, then ``offset`` records
            are skipped before starting to count the limit records that are
            returned.


        @return: An object containing the following attributes:

        """
        return Object.from_json(
            self.send_request(
                http_method=self.HttpMethod.GET,
                path='/child',
                arguments={
                    'keywords': keywords and ','.join(keywords if isinstance(keywords, (list, set, tuple)) else [keywords]),
                    'limit': limit,
                    'locale': locale,
                    'offset': offset
                },
                authentication_required=True,
                signature_required=True))



# Copyright (C) 2020 Majormode.  All rights reserved.
#
# This software is the confidential and proprietary information of
# Majormode or one of its subsidiaries.  You shall not disclose this
# confidential information and shall use it only in accordance with the
# terms of the license agreement or other applicable agreement you
# entered into with Majormode.
#
# MAJORMODE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
# OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
# TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE, OR NON-INFRINGEMENT.  MAJORMODE SHALL NOT BE LIABLE FOR ANY
# LOSSES OR DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
# OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.

from majormode.perseus.client.service.base_service import BaseService
from majormode.perseus.model.locale import DEFAULT_LOCALE
from majormode.perseus.model.obj import Object


class BusRouteService(BaseService):
    BaseService._declare_custom_exceptions({
    })

    def accept_bus_stop_suggestion(
            self,
            suggestion_id,
            bus_stop=None):
        """
        Accept a bus stop suggestion that the server platform automatically
        made based on the Ci/Co operation activities of the pupils of a school.


        :param suggestion_id: Identification of the bus stop suggestion.

        :param bus_stop: An object `BusStop` that provides the information of
            the bus stop to create.  This argument MUST be passed when the
            suggestion corresponds to the creation of a bus stop.


        :return: An object containing the following attributes:

            * ``stop_id`` (required): Identification of the bus stop.

            * ``update_time`` (required): Time of the most recent modification of
                the information of this bus stop.
        """
        return Object.from_json(
            self.send_request(
                http_method=self.HttpMethod.POST,
                path='/bus-stop/suggestion/(suggestion_id)/acceptance',
                url_bits={
                    'suggestion_id': suggestion_id
                },
                message_body=bus_stop,
                authentication_required=True,
                signature_required=True))

    def get_bus_routes(
            self,
            school_id,
            include_path=False,
            locale=DEFAULT_LOCALE):
        """
        Return a list of children that have joined the School Bus Program and
        that correspond to the specified criteria.


        :param school_id: Identification of the school to return the list of
            bus routes.

        :param include_path: Indicate whether the path of the route needs to
            be returned.

        :param locale: A `Locale` object to return textual information,
            such as the names of countries and the names of children.


        :return: A list of objects containing the following attributes:

                {
                  "routes": [
                    {
                      "creation_time": timestamp,
                      "stops": [
                        {
                          "stop_id": string,
                          "position": integer,
                          "arrival_time": time,
                          "departure_time": time
                        },
                        ...
                      ],
                      "journey_type": string,
                      "object_status": string,
                      "path": [
                        [latitude, longitude, altitude],
                        ...
                      ],
                      "route_id": string,
                      "update_time": timestamp,
                      "visibility": string
                    },
                    ...
                  ],
                  "stops": [
                    {
                      "address": {
                        component_type: string,
                        ...
                      },
                      "creation_time": timestamp,
                      "locale": string,
                      "location": {
                        "altitude": decimal,
                        "latitude": decimal,
                        "longitude": decimal
                      },
                      "object_status": string,
                      "stop_id": string,
                      "update_time": timestamp
                    }
                  ]
                }

            where:

            * `routes` (required): The list of routes of the school buses:

              * `creation_time` (required): The time when this route has been
                created.

              * `stops` (required): The list of stops along the route:

                * `stop_id` (required): Identification of a bus stop used to board or
                  alight children when taking them to school (outward journey) or taking
                  them back to home from their school (inward journey).

                * `position` (required): Position of the bus stop along this route,
                  starting from `0`.

                * `arrival_time` (optional): Time when the school bus pulls up to the
                  bus stop's area.  This time is possibly not defined for the first bus
                  stop of the route.

                * `departure_time` (optional): Time whe the school bus leaves the bus
                  stop's area.  This time is possibly not defined for the last bus stop
                  of the route.

              * `journey_type`: Indicate the direction of the journey:

                * `outward`: From children's homes to the school.

                * `inward`: From the school to children's homes.

              * `object_status`: Current status of this route.

              * `path` (optional): A list of geographic coordinates defining the
                path of the route through the bus stops.

              * `route_id`: Identification of the route.

              * `update_time`: Time of the most recent modification of an
                information regarding this route.

              * `visibility`: Indicate the visibility to users:

                * `private`: This route is only visible to members of organization
                  responsible for managing the bus transportation service of the school
                  that this route takes children to or from, and parents whose children
                  are registered to this route.

                * `public`: This route is visible to any authenticated or anonymous
                  users.

                * `team`: This route is visible to any users registered to the school
                  that this route takes children to or from.

              * `stops` (required): The list of the places where buses of the school
                stop for children to get on or get off the bus.

              * `address` (optional): Postal address of the place corresponding to
                the bus stop.  This address is composed of one or more address
                components, which textual information is written in the specified
                locale.  An address component is defined with a component type and its
                value (refer to the `Place Service` documentation for the complete
                list of address components).

              * `creation_time` (required): Time when this bus stop has been created.

              * `locale` (required): Locale in which the values of each address
                component of the bus stop are written.

              * `location` (required): Information about the geographic location of
                the bus stop:

                * `altitude` (optional): Altitude in meters of the geographic location
                  of the bus stop.

                * `latitude` (required): Latitude-angular distance, expressed in
                  decimal degrees (WGS84 datum), measured from the center of the Earth,
                  of a point north or south of the Equator corresponding to the
                  geographic location of the bus stop.

                * `longitude` (required): Longitude-angular distance, expressed in
                  decimal degrees (WGS84 datum), measured from the center of the Earth,
                  of a point east or west of the Prime Meridian corresponding to the
                  geographic location of the bus stop.

              * `object_status` (required): Current status of this bus stop.

              * `stop_id` (required): Identification of the bus stop.

              * `update_time` (required): Time of the most recent modification of
                an attribute of the bus stop.
        """
        return Object.from_json(
            self.send_request(
                http_method=self.HttpMethod.GET,
                path='/school/(school_id)/bus-route',
                url_bits={
                    'school_id': school_id
                },
                arguments={
                    'include_path': include_path,
                    'locale': locale
                },
                authentication_required=False,
                signature_required=True))

    def add_bus_line(
            self,
            school_id,
            bus_line):
        return Object.from_json(
            self.send_request(
                http_method=self.HttpMethod.POST,
                path='/school/(school_id)/bus-line',
                url_bits={
                    'school_id': school_id
                },
                message_body=bus_line,
                authentication_required=True,
                signature_required=True))

    def add_bus_stops(
            self,
            school_id,
            bus_stops):
        return Object.from_json(
            self.send_request(
                http_method=self.HttpMethod.POST,
                path='/school/(school_id)/bus-stop',
                url_bits={
                    'school_id': school_id
                },
                message_body=bus_stops,
                authentication_required=True,
                signature_required=True))

    def reject_bus_stop_suggestion(
            self,
            suggestion_id):
        """
        Reject a bus stop suggestion that the server platform automatically
        made based on the Ci/Co operation activities of the pupils of a school.


        :param suggestion_id: Identification of a bus stop suggestion.


        :raise DeletedObjectException: If the bus stop suggestion has been
            already reviewed and deleted.

        :raise DisabledObjectException: If the bus stop suggestion has been
            already reviewed and ignored.

        :raise IllegalAccessException: If the user is not granted access to
            bus stop suggestions of this school.

        :raise UndefinedObjectException: If the bus stop suggestion does not
            exist.
        """
        return Object.from_json(
            self.send_request(
                http_method=self.HttpMethod.POST,
                path='/bus-stop/suggestion/(suggestion_id)/rejection',
                url_bits={
                    'suggestion_id': suggestion_id
                },
                authentication_required=True,
                signature_required=True))



# Copyright (C) 2020 Majormode.  All rights reserved.
#
# This software is the confidential and proprietary information of
# Majormode or one of its subsidiaries.  You shall not disclose this
# confidential information and shall use it only in accordance with the
# terms of the license agreement or other applicable agreement you
# entered into with Majormode.
#
# MAJORMODE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
# OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
# TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE, OR NON-INFRINGEMENT.  MAJORMODE SHALL NOT BE LIABLE FOR ANY
# LOSSES OR DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
# OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.

from majormode.perseus.client.service.base_service import BaseService
from majormode.perseus.model.locale import DEFAULT_LOCALE
from majormode.perseus.model.obj import Object


class AttendantService(BaseService):
    BaseService._declare_custom_exceptions({
    })

    def get_trackers_locations(
            self,
            team_id,
            end_time=None,
            limit=None,
            max_fix_age=None,
            sort_order=None,
            start_time=None):
        """
        Return lists of geographic location updates of tracker mobile devices
        used by the attendants of a school transport organization.

        :note: The user on behalf of whom this function is called must be an
            administrator of the school transport organization.


        :param team_id: Identification of a school transport organization.

        :param end_time: Latest inclusive time to filter location updates
            based on their fix time.

        :param limit: Constrain the number of locations per tracker mobile
            device to return to the specified number.  The default value is
            `AttendantService.DEFAULT_LIMIT`.  The maximum value is
            `AttendantService.MAXIMUM_LIMIT`.

        :param max_fix_age: Maximum difference, expressed in seconds, between
            the time when a tracker mobile device calculated the location fix
            and the current time on the server platform.  This duration
            corresponds to the current age of the location fix.

        :param sort_order: An item of the enumeration `SortOrder` that
            indicates the order to sort location updates:

            * `SortOrder.ascending` (default): The location updates of a tracker
              mobile device are sorted by ascending order of their fix time.

            * `SortOrder.descending`: The location updates of a tracker mobile
              device are sorted by descending order of their fix time.

        :param start_time: Earliest non-inclusive time to filter location
            updates based on their fix time.


        :return: A list of objects containing the following attributes:

            * `account_id` (required): Identification of the account of an
              attendant belonging to the specified school transport organization.

            * `battery_level` (required): Current level in percentage of the
              battery of the tracker mobile device.

            * `device_id` (required): Identification of the tracker mobile device
              currently used by the attendant.

            * `is_battery_plugged` (required): Indicate if the internal battery of
              the tracker mobile device is plugged to a power source.

            * `locations` (optional): A list of geographic locations reported by
              the tracker mobile device:

              * `accuracy` (required): Accuracy in meters of the location.

              * `altitude` (required): Altitude in meters of the location.

              * `bearing` (optional): Bearing in degrees.  Bearing is the horizontal
                direction of travel of the tracker mobile device, and is not related
                to the device orientation.  It is guaranteed to be in the range
                `[0.0, 360.0]`.

              * `fix_time` (required): Time when the tracker mobile device determined
                the information of this fix.

              * `latitude` (required): Latitude-angular distance, expressed in decimal
                degrees (WGS84 datum), measured from the center of the Earth, of a
                point north or south of the Equator corresponding to the location.

              * `longitude` (required): Longitude-angular distance, expressed in
                decimal degrees (WGS84 datum), measured from the center of the Earth,
                of a point east or west of the Prime Meridian corresponding to the
                location.

              * `provider` (required): Type of the location provider that reported
                this geographical location:

                * `gps`: This provider determines location using satellites (Global
                  Positioning System (GPS)).

                * `network`: This provider determines location based on availability of
                  cell towers and Wi-Fi access points.  Results are retrieved by means
                  of a network lookup.

                * `passive`: A special location provider for receiving locations without
                  actually initiating a location fix.  This provider can be used to
                  passively receive location updates when other applications or services
                  request them without actually requesting the locations yourself.  This
                  provider will return locations generated by other providers.

              * `speed` (optional): Speed in meters/second over the ground.


        :raise DeletedObjectException: If the specified team has been deleted.

        :raise DisabledObjectException: If the specified team has been disabled.

        :raise IllegalAccessException: If the account of the user on behalf of
            whom this function is called is not an administrator of the school
             transport organization.

        :raise UndefinedObjectException: If the specified team is not registered.
        """
        return Object.from_json(
            self.send_request(
                http_method=self.HttpMethod.GET,
                path='/attendant/tracker/location',
                arguments={
                    'end_time': end_time,
                    'limit': limit,
                    'max_fix_age': max_fix_age,
                    'sort_order': sort_order,
                    'start_time': start_time,
                    'team_id': team_id,
                },
                authentication_required=True,
                signature_required=True))



# Copyright (C) 2019 Majormode.  All rights reserved.
#
# This software is the confidential and proprietary information of
# Majormode or one of its subsidiaries.  You shall not disclose this
# confidential information and shall use it only in accordance with the
# terms of the license agreement or other applicable agreement you
# entered into with Majormode.
#
# MAJORMODE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
# OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
# TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE, OR NON-INFRINGEMENT.  MAJORMODE SHALL NOT BE LIABLE FOR ANY
# LOSSES OR DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
# OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.

from majormode.perseus.client.service.base_service import BaseService
from majormode.perseus.model.obj import Object


class EducationService(BaseService):
    BaseService._declare_custom_exceptions({
    })

    def get_school_grades(self, country_code):
        """
        Return the list of subdivisions of formal learning for the specified
        country.  They are known as school grades (US), school years or levels
        (UK).


        :param country_code: An ISO 3166-1 alpha-2 code representing the
            country to return school grades from.


        :return: A list of instances containing the following attributes:

            * `grade_level` (required): Number of the year a pupil has reached in
              his given educational stage for this grade

            * `grade_name` (required): Name given to this grade.

            * `start_age` (required): Age at which pupils usually begin this grade.

            * `end_age` (required): Age at which pupils usually complete this grade.
        """
        return Object.from_json(
            self.send_request(
                http_method=self.HttpMethod.GET,
                path='/school_grades/(country_code)',
                url_bits={
                    'country_code': country_code
                },
                authentication_required=False,
                signature_required=True))



# Copyright (C) 2019 Majormode.  All rights reserved.
#
# This software is the confidential and proprietary information of
# Majormode or one of its subsidiaries.  You shall not disclose this
# confidential information and shall use it only in accordance with the
# terms of the license agreement or other applicable agreement you
# entered into with Majormode.
#
# MAJORMODE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
# OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
# TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE, OR NON-INFRINGEMENT.  MAJORMODE SHALL NOT BE LIABLE FOR ANY
# LOSSES OR DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
# OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.

from majormode.perseus.client.service.base_service import BaseService
from majormode.perseus.model.locale import DEFAULT_LOCALE
from majormode.perseus.model.obj import Object


class SchoolService(BaseService):
    BaseService._declare_custom_exceptions({
    })

    def get_bus_stops(
            self,
            school_id,
            include_ignored=False,
            include_suggested=False,
            limit=None,
            locale=None,
            offset=0,
            sync_time=None):
        """
        Return a list of bus stops where school buses pick up and drop off
        children of a school.


        :param school_id: Identification of a school.

        :param include_ignored: Indicate whether to return bus stops that have
            been automatically detected but ignored by an administrator of an
            organization managing the transportation for this school.

        :param include_suggested: Indicate whether to return bus stops which
            changes been automatically detected (new bus stops, bus stops
            which location has slightly changed, unused bus stops).

        :param limit: Constrain the number of bus stops to return to the
            specified number.  If not specified, the default value is `10`.
            The maximum value is `100`.

        :param locale: The locale to return textual information, such as the
             name of the bus stop, its address.  By default, textual
             information is returned in English.

        :param offset: Require to skip that many bus stops before beginning to
            return bus stops to the client.  Default value is `0`.  If both
            `offset` and `limit` are specified, then `offset` bus stops are
            skipped before starting to count the limit bus stops that are
            returned.

        :param sync_time: Earliest time to return bus stops based on the time
            of the most recent modification of one of their attributes.  If
            not specified, the platform returns all the bus stops.


        :return: A list of objects containing the following attributes:

            * `address` (required): Postal address of the school, composed of one or
              more address components, which textual information is written in the
              specified locale. An address component is defined with a component
              type and its value.

            * `location` (required): Geographical location of the bus stop.

              * `altitude` (required): Altitude in meters of the bus stop's location.

              * `latitude` (required): Latitude-angular distance, expressed in decimal
                degrees (WGS84 datum), measured from the center of the Earth, of a
                point north or south of the Equator corresponding to the bus stop's
                location.

              * `longitude` (required): Longitude-angular distance, expressed in
                decimal degrees (WGS84 datum), measured from the center of the Earth,
                of a point east or west of the Prime Meridian corresponding to the bus
                stop's location.

              * `radius` (optional): Radius in meters of the surface area of the
                circle that contains the bus stop.

            * `stop_id` (required): Identification of the bus stops.

            * `update_time` (required): Time of the most recent modification of an
              attribute of this bus stop.
        """
        return Object.from_json(
            self.send_request(
                http_method=self.HttpMethod.GET,
                path='/school/(school_id)/bus-stop',
                url_bits={
                    'school_id': school_id
                },
                arguments={
                    'include_ignored': include_ignored,
                    'include_suggested': include_suggested,
                    'limit': limit,
                    'locale': locale,
                    'offset': offset,
                    'sync_time': sync_time,
                },
                authentication_required=True,
                signature_required=True))

    def get_school_buses_locations(self, school_id):
        """

        :param school_id: Identification of a school.


        :return: A list of objects containing the following attributes:

* `account_id` (required): Identification of the attendant who is using
  the tracker device.

* `accuracy` (optional): Accuracy in meters of the location.

* `altitude` (optional):

* `bearing` (optional): Bearing is the horizontal direction of travel
  of the mobile tracker device, and is not related to the device's
  orientation.  It is guaranteed to be in the range `[0.0, 360.0]`.

* `fix_time` (required): Time when the mobile tracker device determined
  the information of this fix.

* `full_name` (required): Full name by which the attendant is known.

* `latitude` (required):

* `longitude` (required):

* `speed` (optional): Speed in meters/second over the ground.


* `team_id` (required): Identification of the team that the attendant
  belongs to.

* `team_name` (required): Name of the team that the attendant belongs to.
        """
        return Object.from_json(
            self.send_request(
                http_method=self.HttpMethod.GET,
                path='/school/(school_id)/bus/location',
                url_bits={
                    'school_id': school_id,
                },
                authentication_required=True,
                signature_required=True))

    def search_schools(
            self,
            area_id=None,
            include_education_grades=False,
            ip_address=None,
            keywords=None,
            location=None,
            limit=None,
            locale=DEFAULT_LOCALE,
            offset=0):
        """
        Return a list of schools that have joined the School Bus Program and
        that correspond to the specified criteria.


        :param area_id: Identification of a geographical area to return
            schools.

        :param include_education_grades: Indicate whether to include the
            education grades that this school supports.

        :param ip_address: IPv4 address of the machine of the user, a tuple
            consisting of four decimal numbers, each ranging from ``0`` to
            ``255``.

        :param keywords: A list of keywords to search schools for.

        :param limit: Constrain the number of places to return to the
            specified number.  If not specified, the default value is
            ``BaseService.DEFAULT_RESULT_SET_SIZE``.  The maximum value is
            ``BaseService.MAXIMUM_RESULT_SET_LIMIT``.

        :param locale: An instance ``Locale`` to return textual information,
            such as the names of countries and the names of schools.

        :param location: An instance ``GeoPoint`` that indicates the user's
            location.

        :param offset: Require to skip that many records before beginning to
            return records to the client.  Default value is ``0``.  If both
            ``offset`` and ``limit`` are specified, then ``offset`` records
            are skipped before starting to count the limit records that are
            returned.


        @return: An object containing the following attributes:

        """
        return Object.from_json(
            self.send_request(
                http_method=self.HttpMethod.GET,
                path='/school',
                arguments={
                    'area_id': area_id,
                    'include_education_grades': include_education_grades,
                    'ip_address': ip_address and '.'.join([str(byte) for byte in ip_address]),
                    'keywords': keywords and ','.join(keywords if isinstance(keywords, (list, set, tuple)) else [keywords]),
                    'limit': limit,
                    'll': location and f'{location.latitude},{location.longitude}',
                    'locale': locale,
                    'offset': offset,
                },
                authentication_required=False,
                signature_required=True))



# Copyright (C) 2019 Majormode.  All rights reserved.
#
# This software is the confidential and proprietary information of
# Majormode or one of its subsidiaries.  You shall not disclose this
# confidential information and shall use it only in accordance with the
# terms of the license agreement or other applicable agreement you
# entered into with Majormode.
#
# MAJORMODE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
# OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
# TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE, OR NON-INFRINGEMENT.  MAJORMODE SHALL NOT BE LIABLE FOR ANY
# LOSSES OR DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
# OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.

from majormode.perseus.client.service.base_service import BaseService
from majormode.perseus.model.contact import Contact
from majormode.perseus.model.locale import DEFAULT_LOCALE
from majormode.perseus.model.obj import Object


class XebusService(BaseService):
    BaseService._declare_custom_exceptions({
    })

    def activate_account(
            self,
            activation_code,
            fullname,
            password,
            mobile_phone_number):
        """
        Activate the account of a user and automatically sign the user in.


        :param activation_code: Identification of the verification request
            that has been sent to the user.

        :param fullname: Complete full name of the user.

        :param password: Password given by the user to allow him later to
            sign-in with his account.

        :param mobile_phone_number: Mobile phone number of the user.


        :return: a session instance containing the following members:

            * `account_id`: identification of the account of the user.

            * `expiration_time`: time when the login session will expire.

            * `fullname`: complete full name of the user as given by the user
              himself, i.e., untrusted information, or as determined from his
              email address as for a ghost account.

            * `session_id`: identification of the login session of the user.
        """
        return Object.from_json(
            self.send_request(
                http_method=self.HttpMethod.POST,
                path=f'/account/activation/{activation_code}',
                message_body={
                    'fullname': fullname,
                    'mobile_phone_number': mobile_phone_number,
                    'password': password
                },
                authentication_required=False,
                signature_required=True))

    def register_user(
            self,
            contact,
            area_id,
            school_id,
            accuracy=None,
            ip_address=None,
            locale=DEFAULT_LOCALE,
            location=None):
        """
        Register the user to the platform with his preferred language and the
        identification of the geographical area where his kid is living/
        studying in.

        Request the platform to send an email address verification to this
        user.


        :param contact: An instance `Contact` corresponding to the contact
            information of the user.  The function will initiate the process
            to verify this contact information.

        :param area_id: Identification of the geographical area where the kid
            of the user is living in.

        :param school_id: Identification of the school the kid of the user is
            studying at.

        :param accuracy: Accuracy in meters of the location of the user when
            he registered to the service.

        :param ip_address: IPv4 address of the machine of the user when he
            registered to the service.

        :param locale: An instance `Locale` representing the preferred
            language of the user.

        :param location: An instance `GeoPoint` representing the location of
            the user when he registered to the service.
        """
        return Object.from_json(
            self.send_request(
                http_method=self.HttpMethod.POST,
                path='/account/registration',
                message_body={
                    'accuracy': accuracy,
                    'area_id': area_id,
                    'contact_name': contact.name,
                    'contact_value': contact.value,
                    'ip_address': ip_address and '.'.join([str(byte) for byte in ip_address]),
                    'll': location and f'{location.latitude},{location.longitude}',
                    'locale': locale,
                    'school_id': school_id
                },
                authentication_required=False,
                signature_required=True))

    def sign_in(self, email_address, password):
        """
        Sign-in the user to the platform providing an email address of the
        user and his password.


        :param email_address: email address of the user.

        :param password: password of the user.


        :return: a session instance containing the following members:

            * `account_id`: identification of the account of the user.

            * `expiration_time`: time when the login session will expire.

            * `fullname`: complete full name of the user as given by the user
              himself, i.e., untrusted information, or as determined from his
              email address as for a ghost account.

            * `is_verified`: indicate whether this contact information has been
              verified, whether it has been grabbed from a trusted Social Networking
              Service (SNS), or whether through a challenge/response process.  The
              user should be reminded to confirm his contact information if not
              already verified, or the user would take the chance to have his
              account suspended.

            * `is_primary`: indicate whether this contact information is the
              primary one for the given property.

            * `session_id`: identification of the login session of the user.

            * `username`: also known as screen name or nickname, username is
              chosen by the end user to identify himself when accessing the platform
              and communicating with others online.  A username should be totally
              made-up pseudonym, not reveal the real name of the person.  The
              username is unique across the platform.  A username is not case
              sensitive.


        :raise DeletedObjectException: if the user account has been deleted.

        :raise DisabledObjectException: if the user account has been disabled.

        :raise AuthenticationFailureException: if the given contact
            information and/or password don't match any account registered
            against the platform.
        """
        payload = Object.from_json(
            self.send_request(
                http_method=self.HttpMethod.POST,
                path='/account/session',
                message_body={
                    'email_address': email_address,
                    'password': password
                },
                authentication_required=False,
                signature_required=True))

        self.set_session(self.session.connection.build_session(payload))

        return self.session



# Copyright (C) 2019 Majormode.  All rights reserved.
#
# This software is the confidential and proprietary information of
# Majormode or one of its subsidiaries.  You shall not disclose this
# confidential information and shall use it only in accordance with the
# terms of the license agreement or other applicable agreement you
# entered into with Majormode.
#
# MAJORMODE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
# OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
# TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE, OR NON-INFRINGEMENT.  MAJORMODE SHALL NOT BE LIABLE FOR ANY
# LOSSES OR DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
# OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.

from majormode.perseus.client.service.base_service import BaseService
from majormode.perseus.model.locale import DEFAULT_LOCALE
from majormode.perseus.model.obj import Object


class GuardianService(BaseService):
    BaseService._declare_custom_exceptions({
    })

    def get_children(
            self,
            limit=None,
            offset=0):
        """
        Return a list of the children of the specified user (either a legal
        guardian or a childminder).


        :param limit: Constrain the number of records to return to the
            specified number.  If not specified, the default value is
            `BaseService.DEFAULT_RESULT_SET_SIZE`.  The maximum value is
            `BaseService.MAXIMUM_RESULT_SET_LIMIT`.

        :param offset: Require to skip that many records before beginning to
            return records to the client.  Default value is `0`.  If both
            `offset` and `limit` are specified, then `offset` records
            are skipped before starting to count the limit records that are
            returned.


        :return: A list of objects containing the following attributes:

            * `account_id` (required): Identification of the account of a user.

            * `fullname` (required): Personal name by which the user is known.

            * `locale` (optional): A `Locale` object that identifies the
              preferred language of the user.

            * `picture_id` (optional): Identification of the user's picture.

            * `picture_url` (optional): Uniform Resource Locator (URL) that
              specifies the location of the user's picture.

            * `place_id` (optional): Identification of the residence of the parent
              where he puts his child up.  Guardian having the role of parent MUST
              have a residence.

            * `role` (required): Role of this guardian towards the child.

            * `update_time` (required)): Time when the information of this user
              account has been updated for the last time.
        """
        return Object.from_json(
            self.send_request(
                http_method=self.HttpMethod.GET,
                path='/guardian/children',
                arguments={
                    'limit': limit,
                    'offset': offset
                },
                authentication_required=True,
                signature_required=True))

    def get_children_location(self):
        """
        Return the last known location of the guardian's children.

        This function retrieves the list of the guardian's children who are
        currently checked-in in a school bus, and return the location of the
        ID-R devices that were used to check-in these children.


        :return: A list of object containing the following attributes:

            * `account_id` (required): Identification of a child.

            * `accuracy` (required): Accuracy in meters of the last known location
              of the child.

            * `altitude` (optional): Altitude in meters of the last known location
              of the child.

            * `attendant_account_id` (required): Identification of the attendant who
              checked-in the child.

            * `bearing` (optional): Bearing in degrees of the last known location of
              the child.  Bearing is the horizontal direction of travel of the
              child, and is not related to the child orientation.  It is guaranteed
              to be in the range `[0.0, 360.0]`, or not given if the bearing of the
              child at its last known location is not provided.

            * `bus_id` (optional): Identification of the school bus where the child
              has been checked-in.

            * `cico_time` (required): Time when the child has been checked-in.

            * `device_id` (required): Identification of the ID-R device that was
              used to check-in the child.

            * `fix_time` (required): Time when the tracker mobile device determined
              the information of this last known location of the child.

            * `latitude` (required): Latitude-angular distance, expressed in decimal
              degrees (WGS84 datum), measured from the center of the Earth, of a
              point north or south of the Equator corresponding to the last known
              location of the child.

            * `longitude` (required): Longitude-angular distance, expressed in
              decimal degrees (WGS84 datum), measured from the center of the Earth,
              of a point east or west of the Prime Meridian corresponding to the
              last known location of the child.

            * `line_id` (optional): Identification of the school bus line for which
              the child has been checked-in.

            * `speed` (optional): Speed in meters/second over the ground.

            * `stop_id` (optional): Identification of the bus stop where the child
              boarded school bus when the attendant checked-in this child.
        """
        return Object.from_json(
            self.send_request(
                http_method=self.HttpMethod.GET,
                path='/guardian/children/location',
                authentication_required=True,
                signature_required=True))

    def register_child(
            self,
            fullname,
            dob,
            # photo_file,
            school_id,
            grade_level,
            first_name=None,
            get_off_school_bus_unaccompanied=False,
            home_address=None,
            home_id=None,
            last_name=None,
            leave_school_compound_unaccompanied=False):
        """
        Register a child to the bus transportation program of the specified
        school, on behalf of a legal guardian of this child.


        :param fullname: Personal name by which the child is known, generally
            composed of first name, middle names, and surname, written in a
            order that depends on the language/culture of the child.

        :param dob: Date of birth of the child.

        :param photo_file: A file-like object corresponding to the identity
            photo of the child.

        :param school_id: Identification of the school this child attends.

        :param grade_level: Number of the year the child has reached in the
            educational stage of the school.

        :param first_name: Forename (also known as *given name*) of the child.
            The first name of a child is used to alphabetically sort a list of
            guardians.

        :param get_off_school_bus_unaccompanied: Indicate whether the child is
            authorized to alight from the school bus that brings him home
            without the presence of a guardian of him (e.g., legal guardian or
            childminder).

        :param home_address: Information about the child's home.  This
            argument is optional, and is ignored, if the argument `home_id` is
            passed.  This argument corresponds to an object containing the
            following attributes:

            * `formatted_address` (required): Postal address of the child's home as
              entered by the user.

            * `geocoded_address` (required): Human-readable postal address of the
              child's home determined by a geocoder from the geographical location
              of the child's residence that the user has specified by clicking on
              the map and adjusting the position of the marker representing the
              child's residence.

            * `is_address_edited` (optional): Indicate whether the user has
              manually edited the postal address of the child's home.

            * `is_location_edited` (optional): Indicate whether the user has
              manually adjusted the position of the marker that identifies the
              location of the child's home.

            * `locale` (required): An object `Locale` of the language in which
              the values of each address component of the child's home are written.
              If not defined, a default locale is used (more likely American English).

            * `location` (required): An object `GeoPoint` of the geographical
              location of the child's home.

        :param home_id: Identification of the child's home, if already defined.
            This argument is optional if the attribute `home_address` is
            passed.  This argument takes precedence over the attribute
            `home_address` if both are passed.

        :param last_name: Surname (also known as *family name*) of the child.
            The last name of a child is used to alphabetically sort a list of
            guardians.

        :param leave_school_compound_unaccompanied: Indicate whether the child is
            authorized to leave school at the end without the presence of a
            guardian of him (e.g., legal guardian or childminder).


        :return: An object containing the following attributes:

            * `account_id` (required): The identification of the user account of
              this child.

            * `file_name` (required): The original local file name of the child's
              identity photo.

            * `picture_id` (required): Identification of the identity photo of the
              child.

            * `picture_url` (required): Uniform Resource Locator (URL) that
              specifies the location of the child's identity photo.  The client
              application can use this URL and append the query parameter `size`
              to specify a given pixel resolution of the photo, such as `thumbnail`,
              `small`, `medium`, or `large`.

            * `update_time` (required): Time of the most recent modification of the
              child's account attributes.  This information SHOULD be stored by the
              client application to manage its cache of guardians.
        """
        return Object.from_json(
            self.send_request(
                http_method=self.HttpMethod.POST,
                path='/guardianship/child',
                message_body={
                    'dob': dob,
                    'first_name': first_name,
                    'fullname': fullname,
                    'get_off_school_bus_unaccompanied': get_off_school_bus_unaccompanied,
                    'grade_level': grade_level,
                    'home_address': home_address and {
                        'formatted_address': home_address.formatted_address,
                        'geocoded_address': home_address.geocoded_address,
                        'is_address_edited': home_address.is_address_edited,
                        'is_location_edited': home_address.is_location_edited,
                        'locale': home_address.locale,
                        'location': home_address.location and
                            f'{home_address.location.latitude},{home_address.location.longitude}',
                    },
                    'home_id': home_id,
                    'last_name': last_name,
                    'leave_school_compound_unaccompanied': leave_school_compound_unaccompanied,
                    'school_id': school_id,
                },
                # files=[photo_file],
                authentication_required=True,
                signature_required=True))

    def request_guardianship(
            self,
            child_id):
        """
        Request the guardianship towards a child on behalf of the
        authenticated user.

        Guardianship request is required when a child has been previously
        registered by another user.  The latter is the default parent for this
        child.


        :param child_id: Identification of the account of a child whose
            guardianship is requested.
        """
        return Object.from_json(
            self.send_request(
                http_method=self.HttpMethod.POST,
                path='/child/(child_id)/guardianship/request',
                url_bits={
                    'child_id': child_id
                },
                authentication_required=True,
                signature_required=True))

    def search_guardians(
            self,
            keywords=None,
            limit=None,
            locale=DEFAULT_LOCALE,
            offset=0):
        """
        Search for users who could be considered as the guardians of a child
        and whose personal names best match the specified words.

        The function returns users whose set of names (also known as name-
        group, e.g. first name, last name, middle name, etc.) start with one
        or more of these words.


        :param keywords: A list of keywords to search guardians for.

        :param limit: Constrain the number of places to return to the
            specified number.  If not specified, the default value is
            ``BaseService.DEFAULT_RESULT_SET_SIZE``.  The maximum value is
            ``BaseService.MAXIMUM_RESULT_SET_LIMIT``.

        :param locale: An object ``Locale`` to return textual information,
            such as the names of countries and the names of guardians.

        :param offset: Require to skip that many records before beginning to
            return records to the client.  Default value is ``0``.  If both
            ``offset`` and ``limit`` are specified, then ``offset`` records
            are skipped before starting to count the limit records that are
            returned.


        @return: An object containing the following attributes:

        """
        return Object.from_json(
            self.send_request(
                http_method=self.HttpMethod.GET,
                path='/guardian',
                arguments={
                    'keywords': keywords and ','.join(keywords if isobject(keywords, (list, set, tuple)) else [keywords]),
                    'limit': limit,
                    'locale': locale,
                    'offset': offset
                },
                authentication_required=True,
                signature_required=True))

    def upload_child_photo(self, child_account_id, file):
        """
        Upload the photo identity of a child on behalf of a legal guardian of
        this child.


        :param child_account_id: Identification of the account of a child.

        :param file: A file-like object to upload to the platform.


        :return: An instance containing the following attributes:

            * ``file_name`` (required): The original local file name as the
            ``filename`` parameter of the ``Content-Disposition`` header.

            * ``picture_id`` (required): Identification of the new avatar of the
                user's account as registered to the platform.

            * ``picture_url`` (required): Uniform Resource Locator (URL) that
              specifies the location of the user's avatar.  The client application
              can use this URL and append the query parameter ``size`` to specify
              a given pixel resolution of the photo, such as ``thumbnail``,
              ``small``, ``medium``, or ``large``.

            * ``update_time`` (required): Time of the most recent modification of
              the attributes of the user's account.  This information should be
              stored by the client application to manage its cache of user
              accounts.


        :raise DeletedObjectException: If the account of the child has been
            deleted.

        :raise DisabledObjectException: If the account of the child has been
            disabled.

        :raise IllegalAccessException: If the user on behalf of whom the
            child's photo identify is uploaded to the platform is not a legal
            guardian of the child, or if the photo has been already added to
            the platform but to another child

        :raise InvalidOperation: If the format of the uploaded image is not
            supported.

        :raise OutdatedPhotoException: If the uploaded photo has been already
            used by the past, and it has been assessed as outdated.

        :raise UndefinedObjectException: If the specified identification
            doesn't refer to the account of a child registered to the platform.
        """
        return Object.from_json(
            self.send_request(
                http_method=self.HttpMethod.PUT,
                path=f'/guardian/child/{child_account_id}/photo',
                files=[file],
                authentication_required=True,
                signature_required=True))



# Copyright (C) 2019 Majormode.  All rights reserved.
#
# This software is the confidential and proprietary information of
# Majormode or one of its subsidiaries.  You shall not disclose this
# confidential information and shall use it only in accordance with the
# terms of the license agreement or other applicable agreement you
# entered into with Majormode.
#
# MAJORMODE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
# OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
# TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE, OR NON-INFRINGEMENT.  MAJORMODE SHALL NOT BE LIABLE FOR ANY
# LOSSES OR DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
# OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.

from majormode.perseus.client.service.base_service import BaseService
from majormode.perseus.model.obj import Object


class UpmdService(BaseService):
    BaseService._declare_custom_exceptions({
    })

    def get_registration(self, registration_id):
        return Object.from_json(
            self.send_request(
                http_method=self.HttpMethod.GET,
                path=f'/upmd/registration/{registration_id}',
                authentication_required=False,
                signature_required=True))
